import sys
import time
import numpy
import logging
import threading

from collections import deque

logger = logging.getLogger(__name__)


class Primitive(object):
    """ A Primitive is an elementary behavior that can easily be combined to create more complex behaviors.

        A primitive is basically a thread with access to a "fake" robot to ensure a sort of sandboxing. More precisely, it means that the primitives will be able to:

        * request values from the real robot (motor values, sensors or attached primitives)
        * request modification of motor values (those calls will automatically be combined among all primitives by the :class:`~pypot.primitive.manager.PrimitiveManager`).

        The syntax of those requests directly match the equivalent code that you could write from the :class:`~pypot.robot.robot.Robot`. For instance you can write::

            class MyPrimitive(Primitive):
                def run(self):
                    while True:
                        for m in self.robot.motors:
                            m.goal_position = m.present_position + 10

                    time.sleep(1)

        .. warning:: In the example above, while it seems that you are setting a new goal_position, you are only requesting it. In particular, another primitive could request another goal_position and the result will be the combination of both request. For example, if you have two primitives: one setting the goal_position to 10 and the other setting the goal_position to -20, the real goal_position will be set to -5 (by default the mean of all request is used, see the :class:`~pypot.primitive.manager.PrimitiveManager` class for details).

        Primitives were developed to allow for the creation of complex behaviors such as walking. You could imagine - and this is what is actually done on the Poppy robot - having one primitive for the walking gait, another for the balance and another for handling falls.

        .. note:: This class should always be extended to define your particular behavior in the :meth:`~pypot.primitive.primitive.Primitive.run` method.

        """
    def __init__(self, robot, *args, **kwargs):
        """ At instanciation, it automatically transforms the :class:`~pypot.robot.robot.Robot`
            into a :class:`~pypot.primitive.primitive.MockupRobot`.

            :param args: the arguments are automatically passed to the run method.
            :param kwargs: the keywords arguments are automatically passed to the run method.

            .. warning:: You should not directly pass motors as argument to the primitive. If you need to, use the method :meth:`~pypot.primitive.primitive.Primitive.get_mockup_motor` to transform them into "fake" motors. See the :ref:`write_own_prim` section for details.

            """
        self.robot = MockupRobot(robot)

        self.args = args
        self.kwargs = kwargs

        self._started = threading.Event()
        self._stop = threading.Event()
        self._resume = threading.Event()

        self._synced = threading.Event()

    def setup(self):
        """ Setup methods called before the run loop.

        You can override this method to setup the environment needed by your primitive before the run loop. This method will be called every time the primitive is started/restarted.
        """
        pass

    def teardown(self):
        """ Tear down methods called after the run loop.

        You can override this method to clean up the environment needed by your primitive. This method will be called every time the primitive is stopped.
        """
        pass

    def _wrapped_run(self):
        logger.info("Primitive %s setup.", self)
        self.setup()

        self.robot._primitive_manager.add(self)

        self.t0 = time.time()
        self._started.set()

        self.run(*self.args, **self.kwargs)

        # Forces a last synced to make sure that all values sent
        # Within the primitives will be sent to the motors.
        self._synced.clear()
        self._synced.wait()

        self.robot._primitive_manager.remove(self)

        logger.info("Primitive %s teardown.", self)
        self.teardown()

    def run(self, *args, **kwargs):
        """ Run method of the primitive thread. You should always overwrite this method.

            :param args: the arguments passed to the constructor are automatically passed to this method
            :param kwargs: the arguments passed to the constructor are automatically passed to this method

            .. warning:: You are responsible of handling the :meth:`~primitive.primitive.Primitive.should_stop`, :meth:`~primitive.primitive.Primitive.should_pause` and :meth:`~primitive.primitive.Primitive.wait_to_resume` methods correctly so the code inside your run function matches the desired behavior. You can refer to the code of the :meth:`~primitive.primitive.LoopPrimitive.run` method of the :class:`~primitive.primitive.LoopPrimitive` as an example.

            After termination of the run function, the primitive will automatically be removed from the list of active primitives of the :class:`~pypot.primitive.manager.PrimitiveManager`.

            """
        pass

    @property
    def elapsed_time(self):
        """ Elapsed time (in seconds) since the primitive runs. """
        return time.time() - self.t0

    # MARK: - Start/Stop handling

    def start(self):
        """ Start or restart (the :meth:`~pypot.primitive.primitive.Primitive.stop` method will automatically be called) the primitive. """
        if self.is_alive():
            self.stop()
            self.wait_to_stop()

        self._started.clear()
        self._resume.set()
        self._stop.clear()
        self._synced.clear()

        self._thread = threading.Thread(target=self._wrapped_run)
        self._thread.daemon = True
        self._thread.start()

        self._started.wait()

        logger.info("Primitive %s started.", self)

    def stop(self):
        """ Requests the primitive to stop. """
        self._resume.set()
        self._stop.set()
        logger.info("Primitive %s stopped.", self)

    def should_stop(self):
        """ Signals if the primitive should be stopped or not. """
        return self._stop.is_set()

    def wait_to_stop(self):
        """ Wait until the primitive actually stops. """
        if hasattr(self, '_thread'):
            self._thread.join()

    def is_alive(self):
        """ Determines whether the primitive is running or not.

            The value will be true only when the :meth:`~primitive.primitive.Primitive.run` function is executed.

            """
        return hasattr(self, '_thread') and self._thread.is_alive()

    # MARK: - Pause/Resume handling

    def pause(self):
        """ Requests the primitives to pause. """
        self._resume.clear()
        logger.info("Primitive %s paused.", self)

    def resume(self):
        """ Requests the primitives to resume. """
        self._resume.set()
        logger.info("Primitive %s resumed.", self)

    def should_pause(self):
        """ Signals if the primitive should be paused or not. """
        return not self._resume.is_set()

    def wait_to_resume(self):
        """ Waits until the primitive is resumed. """
        self._resume.wait()

    def get_mockup_motor(self, motor):
        """ Gets the equivalent :class:`~pypot.primitive.primitive.MockupMotor`. """
        return next((m for m in self.robot.motors if m.name == motor.name), None)


class LoopPrimitive(Primitive):
    """ Simple primitive that call an update method at a predefined frequency.

        You should write your own subclass where you only defined the :meth:`~pypot.primitive.primitive.Primitive.update` method.

        """
    def __init__(self, robot, freq, *args, **kwargs):
        Primitive.__init__(self, robot, *args, **kwargs)
        self._period = 1.0 / freq
        self._recent_updates = deque([], 11)

    @property
    def recent_update_frequencies(self):
        """ Returns the 10 most recent update frequencies.

        The given frequencies are computed as short-term frequencies!
        The 0th element of the list corresponds to the most recent frequency.
        """
        return list(reversed([(1.0 / p) for p in numpy.diff(self._recent_updates)]))

    def run(self, *args, **kwargs):
        """ Calls the :meth:`~pypot.primitive.primitive.Primitive.update` method at a predefined frequency (runs until stopped). """

        while not self.should_stop():
            start = time.time()
            self._wrapped_update(*self.args, **self.kwargs)
            end = time.time()

            self._recent_updates.append(start)

            dt = self._period - (end - start)
            if dt > 0:
                time.sleep(dt)

            if self.should_pause():
                self.wait_to_resume()

    def _wrapped_update(self, *args, **kwargs):
        logger.debug('LoopPrimitive %s updated.', self)
        self.update(*args, **kwargs)

    def update(self, *args, **kwargs):
        """ Update methods that will be called at a predefined frequency.

            :param args: the arguments passed to the constructor are automatically passed to this method
            :param kwargs: the arguments passed to the constructor are automatically passed to this method
            """
        raise NotImplementedError


class MockupRobot(object):
    """ Fake :class:`~pypot.robot.robot.Robot` used by the :class:`~pypot.primitive.primitive.Primitive` to ensure sandboxing. """
    def __init__(self, robot):
        self._robot = robot
        self._motors = []

        for a in robot.alias:
            setattr(self, a, [])

        for m in robot.motors:
            mockup_motor = MockupMotor(m)
            self._motors.append(mockup_motor)
            setattr(self, m.name, mockup_motor)

            for a in [a for a in robot.alias if m in getattr(robot, a)]:
                getattr(self, a).append(mockup_motor)

    def __getattr__(self, attr):
        return getattr(self._robot, attr)

    def goto_position(self, position_for_motors, duration, wait=False):
        for motor_name, position in position_for_motors.iteritems():
            m = getattr(self, motor_name)
            m.goto_position(position, duration)

        if wait:
            time.sleep(duration)

    @property
    def motors(self):
        """ List of all attached :class:`~pypot.primitive.primitive.MockupMotor`. """
        return self._motors

    def power_max(self):
        for m in self.motors:
            m.compliant = False
            m.moving_speed = 0
            m.torque_limit = 100.0


class MockupMotor(object):
    """ Fake Motor used by the primitive to ensure sandboxing:

        * the read instructions are directly delegate to the real motor
        * the write instructions are stored as request waiting to be combined by the primitive manager.

        """
    def __init__(self, motor):
        object.__setattr__(self, '_m', motor)
        object.__setattr__(self, '_to_set', {})

    def __getattr__(self, attr):
        return getattr(self._m, attr)

    def __setattr__(self, attr, val):
        if attr == 'goal_speed':
            MockupMotor.goal_speed.fset(self, val)
        else:
            self._to_set[attr] = val
            logger.debug("Setting MockupMotor '%s.%s' to %s",
                         self.name, attr, val)

    def goto_position(self, position, duration, wait=False):
        """ Automatically sets the goal position and the moving speed to reach the desired position within the duration. """
        dp = abs(self.present_position - position)
        speed = (dp / float(duration)) if duration > 0 else numpy.inf
        self.moving_speed = speed
        self.goal_position = position

        if wait:
            time.sleep(duration)

    @property
    def goal_speed(self):
        """ Goal speed (in degrees per second) of the motor.

            This property can be used to control your motor in speed. Setting a goal speed will automatically change the moving speed and sets the goal position as the angle limit.

            .. note:: The motor will turn until reaching the angle limit. But this is not a wheel mode, so the motor will stop at its limits.

            """
        return numpy.sign(self.goal_position) * self.moving_speed

    @goal_speed.setter
    def goal_speed(self, value):
        if abs(value) < sys.float_info.epsilon:
            self.goal_position = self.present_position

        else:
            # 0.7 corresponds approx. to the min speed that will be converted into 0
            # and as 0 corredsponds to setting the max speed, we have to check this case
            value = numpy.sign(value) * 0.7 if abs(value) < 0.7 else value

            self.goal_position = numpy.sign(value) * self.max_pos
            self.moving_speed = abs(value)



# -*- coding: utf-8 -*

import sys
import time
import numpy
import logging

from collections import defaultdict
from operator import getitem, setitem


logger = logging.getLogger(__name__)


class DxlMotor(object):
    """ High-level class used to represent and control a generic dynamixel motor.

        This class provides all level access to:
            * motor id
            * motor name
            * position/speed/load (read and write)
            * compliant
            * motor orientation and offset
            * angle limit
            * temperature
            * voltage

        This class represents a generic robotis motor and you define your own subclass for specific motors (see :class:`~pypot.dynamixel.motor.DxlMXMotor` or :class:`~pypot.dynamixel.motor.DxlAXRXMotor`).

        Those properties are synchronized with the real motors values thanks to a :class:`~pypot.dynamixel.controller.DxlController`.

        """
    def __init__(self, id, name=None,
                 direct=True, offset=0.0):
        self._id = id
        self._name = name if name else 'motor_{}'.format(id)

        self._direct = direct
        self._offset = offset

        self._values = defaultdict(int)
        self._values['compliant'] = True

    def __repr__(self):
        return '<DxlMotor name={self.name} id={self.id} pos={self.present_position}>'.format(self=self)

    @property
    def json(self):
        return self.name

    def _setter(self, name, value):
        setitem(self._values, name, value)
        logger.debug("Setting '%s.%s' to %s",
                     self.name, name, value)

    @classmethod
    def _make_accessor(cls, name, rw=False, doc=None):
        def getter(self):
            return getitem(self._values, name)

        def setter(self, value):
            self._setter(name, value)

        return property(fget=getter,
                        fset=setter if rw else None,
                        doc=doc)

    @property
    def id(self):
        """ Id of the motor (readonly). """
        return self._id

    @property
    def name(self):
        """ Name of the motor (readonly). """
        return self._name

    @property
    def present_position(self):
        """ Present position (in degrees) of the motor (readonly). """
        pos = self._values['present_position']
        return (pos if self.direct else -pos) - self.offset

    @property
    def goal_position(self):
        """ Goal position (in degrees) of th motor. """
        pos = self._values['goal_position']
        return (pos if self.direct else -pos) - self.offset

    @goal_position.setter
    def goal_position(self, value):
        framed_value = (value + self.offset) if self.direct else -(value + self.offset)
        self._values['goal_position'] = framed_value

        logger.debug("Setting '%s.goal_position' to %s (%s)",
                     self.name, value, framed_value)

    @property
    def present_speed(self):
        """ Present speed (in degrees per second) of the motor (readonly). """
        speed = self._values['present_speed']
        return (speed if self.direct else -speed)

    @property
    def goal_speed(self):
        """ Goal speed (in degrees per second) of the motor.

            This property can be used to control your motor in speed. Setting a goal speed will automatically change the moving speed and sets the goal position as the angle limit.

            .. note:: The motor will turn until reaching the angle limit. But this is not a wheel mode, so the motor will stop at its limits.

            """
        return numpy.sign(self.goal_position) * self.moving_speed

    @goal_speed.setter
    def goal_speed(self, value):
        if abs(value) < sys.float_info.epsilon:
            self.goal_position = self.present_position

        else:
            # 0.7 corresponds approx. to the min speed that will be converted into 0
            # and as 0 corredsponds to setting the max speed, we have to check this case
            value = numpy.sign(value) * 0.7 if abs(value) < 0.7 else value

            self.goal_position = numpy.sign(value) * self.max_pos
            self.moving_speed = abs(value)

    @property
    def present_load(self):
        """ Present load (in percentage of max load) of the motor (readonly). """
        load = self._values['present_load']
        return (load if self.direct else -load)

    @property
    def compliant(self):
        """ Compliancy of the motor. """
        return bool(self._values['compliant'])

    @compliant.setter
    def compliant(self, value):
        # Change the goal_position only if you switch from compliant to not compliant mode
        if not value and self.compliant:
            self.goal_position = self.present_position
        self._setter('compliant', value)

    @property
    def direct(self):
        """ Orientation of the motor. """
        return self._direct

    @property
    def offset(self):
        """ Offset of the zero of the motor (in degrees). """
        return self._offset

    @property
    def registers(self):
        return [r for r in dir(self) if not r.startswith('_')]

    def goto_position(self, position, duration, wait=False):
        """ Automatically sets the goal position and the moving speed to reach the desired position within the duration. """
        dp = abs(self.present_position - position)
        speed = (dp / float(duration)) if duration > 0 else numpy.inf

        self.moving_speed = speed
        self.goal_position = position

        if wait:
            time.sleep(duration)


DxlMotor.moving_speed = DxlMotor._make_accessor('moving_speed', rw=True,
                                                doc='Moving speed (in degrees per second) of the motor.')
DxlMotor.torque_limit = DxlMotor._make_accessor('torque_limit', rw=True,
                                                doc='Torque limit (in percentage of max torque) of the motor.')

DxlMotor.angle_limit = DxlMotor._make_accessor('angle_limit',
                                               doc='Angle limit (in degrees) of the motor.')
DxlMotor.present_voltage = DxlMotor._make_accessor('present_voltage',
                                                   doc='Present voltage (in V) of the motor.')
DxlMotor.present_temperature = DxlMotor._make_accessor('present_temperature',
                                                       doc='Present temperature (in °C) of the motor.')


class DxlAXRXMotor(DxlMotor):
    """ This class represents the AX robotis motor.

        This class adds access to:
            * compliance margin/slope (see the robotis website for details)

        """
    def __init__(self, id, name=None,
                 direct=True, offset=0.0):
        DxlMotor.__init__(self, id, name, direct, offset)
        self.max_pos = 150


DxlAXRXMotor.compliance_margin = DxlAXRXMotor._make_accessor('compliance_margin', rw=True,
                                                             doc='Compliance margin of the motor (see robotis website).')
DxlAXRXMotor.compliance_slope = DxlAXRXMotor._make_accessor('compliance_slope', rw=True,
                                                            doc='Compliance slope of the motor (see robotis website).')


class DxlMXMotor(DxlMotor):
    def __init__(self, id, name=None,
                 direct=True, offset=0.0):
        """ This class represents the RX and MX robotis motor.

            This class adds access to:
                * PID gains (see the robotis website for details)

            """
        DxlMotor.__init__(self, id, name, direct, offset)
        self.max_pos = 180

DxlMXMotor.pid = DxlMXMotor._make_accessor('pid', rw=True,
                                           doc='PID gains of the motors (see robotis website).')



import logging

from collections import defaultdict

from ..utils import attrgetter, attrsetter


logger = logging.getLogger(__name__)


class BaseRequestHandler(object):
    """ JSON request (get, set or call) handler. """

    def __init__(self, robot):
        self.robot = robot

    def handle_request(self, request):
        """ Handles a JSON request from the client.

            The request can contain at the same time a get, a set and a call request. Each of these requests can retrieve (resp. affect and call) multiple values at the same time, so for instance you can get the position, speed, load and temperature of multiple motors and get the pressure value of a sensor in the same request.

            The request are constructed as follows::

                {
                    'get': { ... },
                    'set': { ... },
                    'call': { ... }
                }

            .. note:: It is not mandatory to give a get, a set and a call request, you can build a request with only a get field.

            Each field of the request will be detailed in their respective handler.

            """
        answer = {}

        for meth in [meth for meth in ('get', 'set', 'call') if meth in request]:
            f = getattr(self, 'handle_{}'.format(meth))
            answer[meth] = f(request[meth])

        logger.debug('Handling request %s with answer %s', request, answer)

        return answer

    def handle_get(self, request):
        """ Handles the get field of a request.

            The get field is constructed as follows::

                {
                    'get': {
                        'obj_name_1': ('var_path_1', 'var_path_2', ...),
                        'obj_name_2': ('var_path_1', 'var_path_2', ...),
                        # ...
                        }
                    }

            For each couple obj_name_1, var_path_1, the handler will get the value corresponding to self.robot.obj_name_1.var_path_1: i.e. if we want to get the present_position and present_speed of the motor named l_knee_y we would use::

                {
                    'get': {
                        'l_knee_y': ('present_position', 'present_speed'),
                        }
                    }

            .. note:: The var_path could be complete path, e.g. "skeleton.left_foot.position.x".

            """
        answer = defaultdict(dict)

        for obj_name, paths in request.iteritems():
            obj = getattr(self.robot, obj_name) if obj_name else self.robot

            for var_path in paths:
                answer[obj_name][var_path] = attrgetter(var_path)(obj)

        return answer

    def handle_set(self, request):
        """ Handles the set field of a request.

            The set field is constructed as follows::

                {
                    'set': {
                        'obj_name_1': {'var_path_1': value,
                                        'var_path_2': value,
                                        # ...
                                        },
                        'obj_name_2': {'var_path_1': value,
                                        'var_path_2': value,
                                       # ...
                                        },
                        # ...
                    }
                }

            For each triple obj_name_1, var_path_1, value the handler will set the corresponding value to self.robot.obj_name_1.var_path_1: i.e. if we want to set the goal_position and goal_speed of the motor named l_knee_y to resp. 0 and 100 we would use::

                {
                    'set': {
                        'l_knee_y': {'goal_position': 0, 'present_speed': 100},
                        }
                    }

            .. note:: The var_path could be complete path, e.g. "skeleton.left_foot.position.x".

            """
        for obj_name, value_pairs in request.iteritems():
            obj = getattr(self.robot, obj_name)

            for var_path, value in value_pairs.iteritems():
                attrsetter(var_path)(obj, value)

    def handle_call(self, request):
        """ Handles the call field of a request.

            The call field is constructed as follows::

                'call': {
                    'obj_name_1': {'meth_path_1': (arg1, arg2, ...),
                                    'meth_path_2': (arg1, arg2, ...),
                                    ...
                                    },

                    'obj_name_2': {'meth_path_1': (arg1, arg2, ...),
                                    'meth_path_2': (arg1, arg2, ...),
                                    ...
                                    },
                        ...
                        }

            For each tuple obj_name_1, meth_path_1 and args  the handler will call the corresponding method of the object with the args: i.e. if we want to call the switch_mode method of a primitive named 'balance' to switch to the third mode we would use::

                'call': {
                    'balance': {'switch_mode': (3, )}
                    }

            .. note:: To call a method without argument juste use an empty list as args::

                'call': {'balance': {'start': ()}}

            """
        answer = defaultdict(dict)

        for obj_name, meth_pairs in request.iteritems():
            obj = getattr(self.robot, obj_name)

            for meth, args in meth_pairs.iteritems():
                f = attrgetter(meth)(obj)
                answer[obj_name][meth] = f(*args) if args else f()

        return answer



import platform
import glob

from .io import DxlIO, DxlError
from .error import BaseErrorHandler
from .controller import DxlController


def get_available_ports():
    """ Tries to find the available usb2serial port on your system. """
    if platform.system() == 'Darwin':
        return glob.glob('/dev/tty.usb*')

    elif platform.system() == 'Linux':
        return glob.glob('/dev/ttyACM*') + glob.glob('/dev/ttyUSB*')

    elif platform.system() == 'Windows':
        import _winreg
        import itertools

        ports = []
        path = 'HARDWARE\\DEVICEMAP\\SERIALCOMM'
        key = _winreg.OpenKey(_winreg.HKEY_LOCAL_MACHINE, path)

        for i in itertools.count():
            try:
                ports.append(str(_winreg.EnumValue(key, i)[1]))
            except WindowsError:
                return ports

    return []


def find_port(ids, strict=True):
    """ Find the port with the specified attached motor ids.

        :param list ids: list of motor ids to find
        :param bool strict: specify if all ids should be find (when set to False, only half motor must be found)

        .. warning:: If two (or more) ports are attached to the same list of motor ids the first match will be returned.

    """
    for port in get_available_ports():
        try:
            with DxlIO(port) as dxl:
                founds = len(dxl.scan(ids))

                if strict and founds == len(ids):
                    return port

                if not strict and founds >= len(ids) / 2:
                    return port
        except DxlError:
            continue

    raise IndexError('No suitable port found for ids {}!'.format(ids))



import zmq
import json
import logging

from .server import AbstractServer, MyJSONEncoder


logger = logging.getLogger(__name__)


class ZMQServer(AbstractServer):
    def __init__(self, robot, host='127.0.0.1', port=8080):
        """ A ZMQServer allowing remote access of a robot instance.

        The server used the REQ/REP zmq pattern. You should always first send a request and then read the answer.

        """
        AbstractServer.__init__(self, robot)

        c = zmq.Context()
        self.socket = c.socket(zmq.REP)
        self.socket.bind('tcp://{}:{}'.format(host, port))

        logger.info('Starting ZMQServer on tcp://%s:%s', host, port)

    def run(self):
        """ Run an infinite REQ/REP loop. """
        while True:
            req = self.socket.recv_json()

            try:
                answer = self.request_handler.handle_request(req)
                self.socket.send(json.dumps(answer, cls=MyJSONEncoder))

            except (AttributeError, TypeError) as e:
                self.socket.send_json({'error': e.message})



"""
    Server provides a remote control of a robot through a primitive.

    The client can access (get or set) all motors, sensors, primitives attached to the robot (as any primitive). The communication between the server and the client follows a predefined JSON protocol (see :mod:`~pypot.server.request` for details).

    So far, two types of server have been implemented:

    * a zmq based server (see :mod:`~pypot.server.zmqserver`) and
    * a html based server (see :mod:`~pypot.server.httpserver`)

    .. note:: All clients lives in the same primitive, so their requests are automatically combined. In future version, each client will spawn its own primitive.

    """

from .zmqserver import ZMQServer
from .httpserver import HTTPServer


import time
import logging

from ..primitive.manager import PrimitiveManager
from ..dynamixel.controller import BaseDxlController


logger = logging.getLogger(__name__)


class Robot(object):
    """
        This class is used to regroup all motors and sensors of your robots.

        Most of the time, you do not want to directly instantiate this class,
        but you rather want to use the factory which creates a robot instance
        from a python dictionnary (see :ref:`config_file`).

        This class encapsulates the different controllers (such as dynamixel ones)
        that automatically synchronize the virtual sensors/effectors instances held
        by the robot class with the real devices. By doing so, each sensor/effector
        can be synchronized at a different frequency.

        This class also provides a generic motors accessor in order to (more or less)
        easily extends this class to other types of motor.

        """
    def __init__(self):
        self._motors = []
        self.alias = []
        self._attached_primitives = {}
        self._dxl_controllers = []
        self._primitive_manager = PrimitiveManager(self.motors)

    def close(self):
        """ Cleans the robot by stopping synchronization and all controllers."""
        self.stop_sync()
        [c.close() for c in self._dxl_controllers]

    def __repr__(self):
        return '<Robot motors={}>'.format(self.motors)

    def _attach_dxl_motors(self, dxl_io, dxl_motors):
        c = BaseDxlController(dxl_io, dxl_motors)
        self._dxl_controllers.append(c)

        for m in dxl_motors:
            setattr(self, m.name, m)

        self._motors.extend(dxl_motors)

    def start_sync(self):
        """ Starts all the synchonization loop (sensor/effector controllers). """
        self._primitive_manager.start()
        [c.start() for c in self._dxl_controllers]

        logger.info('Starting robot synchronization.')

    def stop_sync(self):
        """ Stops all the synchonization loop (sensor/effector controllers). """
        if self._primitive_manager.is_alive():
            self._primitive_manager.stop()
        [c.stop() for c in self._dxl_controllers]

        logger.info('Stopping robot synchronization.')

    def attach_primitive(self, primitive, name):
        setattr(self, name, primitive)
        self._attached_primitives[name] = primitive

        logger.info("Attaching primitive '%s' to the robot.", name)

    @property
    def motors(self):
        """ Returns all the motors attached to the robot. """
        return self._motors

    @property
    def primitives(self):
        """ Returns all the primitives attached to the robot. """
        return self._primitive_manager._prim

    @property
    def attached_primitives(self):
        """ Returns all the primitives name attached to the robot. """
        return self._attached_primitives.values()

    @property
    def attached_primitives_name(self):
        """ Returns all the primitives name attached to the robot. """
        return self._attached_primitives.keys()

    @property
    def compliant(self):
        """ Returns a list of all the compliant motors. """
        return [m for m in self.motors if m.compliant]

    @compliant.setter
    def compliant(self, is_compliant):
        """ Switches all motors to compliant (resp. non compliant) mode. """
        for m in self.motors:
            m.compliant = is_compliant

    def goto_position(self, position_for_motors, duration, wait=False):
        """ Moves a subset of the motors to a position within a specific duration.

            :param dict position_for_motors: which motors you want to move {motor_name: pos, motor_name: pos,...}
            :param float duration: duration of the move
            :param bool wait: whether or not to wait for the end of the move

            .. note::In case of dynamixel motors, the speed is automatically adjusted so the goal position is reached after the chosen duration.

            """
        for motor_name, position in position_for_motors.iteritems():
            m = getattr(self, motor_name)
            m.goto_position(position, duration)

        if wait:
            time.sleep(duration)

    def power_up(self):
        """ Changes all settings to guarantee the motors will be used at their maximum power. """
        for m in self.motors:
            m.compliant = False
            m.moving_speed = 0
            m.torque_limit = 100.0



"""
The config module allows the definition of the structure of your robot.

Configuration are written as Python dictionary so you can define/modify them programmatically. You can also import them form file such as JSON formatted file. In the configuration you have to define:

* controllers: For each defined controller, you can specify the port name, the attached motors and the synchronization mode.
* motors: You specify all motors belonging to your robot. You have to define their id, type, orientation, offset and angle_limit.
* motorgroups: It allows to define alias of group of motors. They can be nested.

"""
import warnings
import logging
import numpy
import time
import json

from ..dynamixel.motor import DxlAXRXMotor, DxlMXMotor
from ..dynamixel.error import BaseErrorHandler
from ..dynamixel.io import DxlIO, DxlError
from ..dynamixel import find_port
from .robot import Robot


ergo_robot_config = {
    'controllers': {
        'my_dxl_controller': {
            'port': 'auto',  # It will automatically try to find the corresponding port depending on the attached motor ids.
            'sync_read': False,
            'attached_motors': ['base', 'head'],  # You can mix motorgroups or individual motors
        },
    },

    'motorgroups': {
        'base': ['base_pan', 'base_tilt_lower', 'base_tilt_upper'],
        'head': ['head_pan', 'head_tilt_lower', 'head_tilt_upper'],
    },

    'motors': {
        'base_pan': {
            'id': 11,
            'type': 'RX-64',
            'orientation': 'direct',
            'offset': 22.5,
            'angle_limit': (-67.5, 112.5),
        },
        'base_tilt_lower': {
            'id': 12,
            'type': 'RX-64',
            'orientation': 'direct',
            'offset': 0.0,
            'angle_limit': (-90.0, 90.0),
        },
        'base_tilt_upper': {
            'id': 13,
            'type': 'RX-64',
            'orientation': 'direct',
            'offset': 0.0,
            'angle_limit': (-90.0, 90.0),
        },
        'head_pan': {
            'id': 14,
            'type': 'RX-28',
            'orientation': 'direct',
            'offset': 22.5,
            'angle_limit': (-67.5, 112.5),
        },
        'head_tilt_lower': {
            'id': 15,
            'type': 'RX-28',
            'orientation': 'indirect',
            'offset': 0.0,
            'angle_limit': (-90.0, 90.0),
        },
        'head_tilt_upper': {
            'id': 16,
            'type': 'RX-28',
            'orientation': 'indirect',
            'offset': 0.0,
            'angle_limit': (-90.0, 90.0),
        },
    },
}
# This logger should always provides the config as extra
logger = logging.getLogger(__name__)


def from_config(config, strict=True):
    """ Returns a :class:`~pypot.robot.robot.Robot` instance created from a configuration dictionnary.

        :param dict config: robot configuration dictionary
        :param bool strict: make sure that all ports, motors are availaible.

        For details on how to write such a configuration dictionnary, you should refer to the section :ref:`config_file`.

        """
    logger.info('Loading config... ', extra={'config': config})
    robot = Robot()

    alias = config['motorgroups']

    # Instatiate the different controllers
    for c_name, c_params in config['controllers'].items():
        port = c_params['port']

        dxl_motors = []
        motor_names = sum([_motor_extractor(alias, name)
                           for name in c_params['attached_motors']], [])
        motor_nodes = map(lambda m: (m, config['motors'][m]), motor_names)
        ids = [params['id'] for _, params in motor_nodes]

        if port == 'auto':
            port = find_port(ids, strict)
            logger.info('Found port {} for ids {}'.format(port, ids))

        dxl_io = DxlIO(port=port,
                       use_sync_read=c_params['sync_read'],
                       error_handler_cls=BaseErrorHandler)

        found_ids = dxl_io.scan(ids)
        if ids != found_ids:
            missing_ids = tuple(set(ids) - set(found_ids))
            msg = 'Could not find the motors {} on bus {}.'.format(missing_ids,
                                                                   dxl_io.port)
            logger.warning(msg)

            if strict:
                raise DxlError(msg)

        # Instatiate the attached motors and set their angle_limits if needed
        changed_angle_limits = {}
        for m_name, m_params in motor_nodes:
            MotorCls = DxlMXMotor if m_params['type'].startswith('MX') else DxlAXRXMotor

            m = MotorCls(id=m_params['id'],
                         name=m_name,
                         direct=True if m_params['orientation'] == 'direct' else False,
                         offset=m_params['offset'])

            dxl_motors.append(m)

            angle_limit = m_params['angle_limit']

            old_limits = dxl_io.get_angle_limit((m.id, ))[0]
            d = numpy.linalg.norm(numpy.asarray(angle_limit) - numpy.asarray(old_limits))

            if d > 1:
                logger.warning("Limits of '%s' changed to %s",
                               m.name, angle_limit,
                               extra={'config': config})
                changed_angle_limits[m.id] = angle_limit

            logger.info("Instantiating motor '%s' id=%d direct=%s limits=%s offset=%s",
                        m.name, m.id, m.direct, angle_limit, m.offset,
                        extra={'config': config})

        logger.info('Instantiating controller on %s with motors %s',
                    c_params['port'], motor_names,
                    extra={'config': config})

        if changed_angle_limits:
            dxl_io.set_angle_limit(changed_angle_limits)
            time.sleep(1)

        robot._attach_dxl_motors(dxl_io, dxl_motors)

    # Create the alias for the motorgroups
    for alias_name in alias:
        motors = [getattr(robot, name) for name in _motor_extractor(alias, alias_name)]
        setattr(robot, alias_name, motors)
        robot.alias.append(alias_name)

        logger.info("Creating alias '%s' for motors %s",
                    alias_name, [motor.name for motor in motors],
                    extra={'config': config})

    logger.info('Loading complete!',
                extra={'config': config})

    return robot


def from_json(json_file):
    """ Returns a :class:`~pypot.robot.robot.Robot` instance created from a JSON configuration file.

    For details on how to write such a configuration file, you should refer to the section :ref:`config_file`.

    """
    with open(json_file) as f:
        config = json.load(f)

    return from_config(config)


def _oldxml_to_config(xml_file):
    msg = 'Using XML file as configuration is deprecated, you should switch to Python dictionnary. You can save them as any format that can directly be transformed into a dictionnary (e.g. json files).'

    warnings.warn(msg, DeprecationWarning)

    import xml.dom.minidom

    config = {}

    dom = xml.dom.minidom.parse(xml_file)
    robot_node = dom.firstChild

    config['controllers'] = {}
    for i, controller_node in enumerate(robot_node.getElementsByTagName('DxlController')):
        name = 'controller_{}'.format(i + 1)
        motors_node = controller_node.getElementsByTagName('DxlMotor')
        config['controllers'][name] = {'port': str(controller_node.getAttribute('port')),
                                       'sync_read': True if controller_node.getAttribute('sync_read') == 'True' else False,
                                       'attached_motors': [m.getAttribute('name') for m in motors_node]}

    config['motorgroups'] = {}
    for motor_group_node in robot_node.getElementsByTagName('DxlMotorGroup'):
        name = str(motor_group_node.getAttribute('name'))
        motors_node = motor_group_node.getElementsByTagName('DxlMotor')
        config['motorgroups'][name] = [str(m.getAttribute('name')) for m in motors_node]

    config['motors'] = {}
    for motor_node in robot_node.getElementsByTagName('DxlMotor'):
        name = str(motor_node.getAttribute('name'))
        angle_limit_node = motor_node.getElementsByTagName('angle_limits')[0]
        angle_limit = eval(angle_limit_node.firstChild.data)
        config['motors'][name] = {'id': int(motor_node.getAttribute('id')),
                                  'type': str(motor_node.getAttribute('type')),
                                  'orientation': str(motor_node.getAttribute('orientation')),
                                  'offset': float(motor_node.getAttribute('offset')),
                                  'angle_limit': angle_limit}

    logger.warning(msg, extra={'config': config})

    return config


def _motor_extractor(alias, name):
    l = []

    if name not in alias:
        return [name]

    for key in alias[name]:
        l += _motor_extractor(alias, key) if key in alias else [key]
    return l



# -*- coding: utf-8 -*-

import serial
import logging
import operator
import itertools
import threading

from collections import namedtuple, OrderedDict
from contextlib import contextmanager

from .conversion import *
from .packet import *

logger = logging.getLogger(__name__)
# With this logger you should always provide as extra:
# - the port
# - the baudrate
# - the timeout


_DxlControl = namedtuple('_DxlControl', ('name',
                                         'address', 'length', 'nb_elem',
                                         'access',
                                         'models',
                                         'dxl_to_si', 'si_to_dxl',
                                         'getter_name', 'setter_name'))


class _DxlAccess(object):
    readonly, writeonly, readwrite = range(3)


class DxlIO(object):
    """ Low-level class to handle the serial communication with the robotis motors. """

    __used_ports = set()
    __controls = []

    # MARK: - Open, Close and Flush the communication

    def __init__(self,
                 port, baudrate=1000000, timeout=0.05,
                 use_sync_read=False,
                 error_handler_cls=None,
                 convert=True):
        """ At instanciation, it opens the serial port and sets the communication parameters.

            :param string port: the serial port to use (e.g. Unix (/dev/tty...), Windows (COM...)).
            :param int baudrate: default for new motors: 57600, for PyPot motors: 1000000
            :param float timeout: read timeout in seconds
            :param bool use_sync_read: whether or not to use the SYNC_READ instruction
            :param error_handler: set a handler that will receive the different errors
            :type error_handler: :py:class:`~pypot.dynamixel.error.DxlErrorHandler`
            :param bool convert: whether or not convert values to units expressed in the standard system

            :raises: :py:exc:`~pypot.dynamixel.io.DxlError` if the port is already used.

            """
        self._known_models = {}
        self._known_mode = {}

        self._sync_read = use_sync_read
        self._error_handler = error_handler_cls() if error_handler_cls else None
        self._convert = convert

        self._serial_lock = threading.Lock()

        self.open(port, baudrate, timeout)

    def __enter__(self):
        return self

    def __del__(self):
        self.close()

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()

    def __repr__(self):
        return ('<DXL IO: closed={self.closed}, '
                'port="{self.port}", '
                'baudrate={self.baudrate}, '
                'timeout={self.timeout}>').format(self=self)

    def open(self, port, baudrate=1000000, timeout=0.05):
        """ Opens a new serial communication (closes the previous communication if needed).

            :raises: :py:exc:`~pypot.dynamixel.io.DxlError` if the port is already used.

            """
        self._open(port, baudrate, timeout)
        logger.info("Opening port '%s'", self.port,
                    extra={'port': port,
                           'baudrate': baudrate,
                           'timeout': timeout})

    def _open(self, port, baudrate, timeout, max_recursion=500):
        # Tries to connect to port until it succeeds to ping any motor on the bus.
        # This is  used to circumvent a bug with the driver for the USB2AX on Mac.
        # Warning: If no motor is connected on the bus, this will run forever!!!
        import platform
        import time

        for i in range(max_recursion):
            self._known_models.clear()
            self._known_mode.clear()

            with self._serial_lock:
                self.close(_force_lock=True)

                if port in self.__used_ports:
                    raise DxlError('port already used {}'.format(port))

                # Dirty walkaround to fix a strange bug.
                # Observed with the USB2AX on Linux with pyserial 2.7
                # We have to first open/close the port in order to make it work at 1Mbauds
                if platform.system() == 'Linux' and self._sync_read:
                    self._serial = serial.Serial(port, 57600)
                    self._serial.close()

                self._serial = serial.Serial(port, baudrate, timeout=timeout)
                self.__used_ports.add(port)

            if platform.system() == 'Darwin' and self._sync_read:
                if not self.ping(DxlBroadcast):
                    self.close()
                    continue
                else:
                    time.sleep(self.timeout)
                    self.flush()
            break
        else:
            raise DxlError('could not connect to the port {}'.format(self.port))

    def close(self, _force_lock=False):
        """ Closes the serial communication if opened. """
        if not self.closed:
            with self.__force_lock(_force_lock) or self._serial_lock:
                self._serial.close()
                self.__used_ports.remove(self.port)

            logger.info("Closing port '%s'", self.port,
                        extra={'port': self.port,
                               'baudrate': self.baudrate,
                               'timeout': self.timeout})

    def flush(self, _force_lock=False):
        """ Flushes the serial communication (both input and output). """
        if self.closed:
            raise DxlError('attempt to flush a closed serial communication')

        with self.__force_lock(_force_lock) or self._serial_lock:
            self._serial.flushInput()
            self._serial.flushOutput()

    def __force_lock(self, condition):
        @contextmanager
        def with_True():
            yield True

        return with_True() if condition else False

    # MARK: Properties of the serial communication

    @property
    def port(self):
        """ Port used by the :class:`~pypot.dynamixel.io.DxlIO`. If set, will re-open a new connection. """
        return self._serial.port

    @port.setter
    def port(self, value):
        self.open(value, self.baudrate, self.timeout)

    @property
    def baudrate(self):
        """ Baudrate used by the :class:`~pypot.dynamixel.io.DxlIO`. If set, will re-open a new connection. """
        return self._serial.baudrate

    @baudrate.setter
    def baudrate(self, value):
        self.open(self.port, value, self.timeout)

    @property
    def timeout(self):
        """ Timeout used by the :class:`~pypot.dynamixel.io.DxlIO`. If set, will re-open a new connection. """
        return self._serial.timeout

    @timeout.setter
    def timeout(self, value):
        self.open(self.port, self.baudrate, value)

    @property
    def closed(self):
        """ Checks if the connection is closed. """
        return not (hasattr(self, '_serial') and self._serial.isOpen())

    # MARK: - Motor discovery

    def ping(self, id):
        """ Pings the motor with the specified id.

            .. note:: The motor id should always be included in [0, 253]. 254 is used for broadcast.

            """
        pp = DxlPingPacket(id)

        try:
            self._send_packet(pp, error_handler=None)
            return True
        except DxlTimeoutError:
            return False

    def scan(self, ids=xrange(254)):
        """ Pings all ids within the specified list, by default it finds all the motors connected to the bus. """
        return [id for id in ids if self.ping(id)]

    # MARK: - Specific Getter / Setter

    def get_model(self, ids):
        """ Gets the model for the specified motors. """
        to_get_ids = [i for i in ids if i not in self._known_models]
        models = [dxl_to_model(m) for m in self._get_model(to_get_ids, convert=False)]
        self._known_models.update(zip(to_get_ids, models))

        return tuple(self._known_models[id] for id in ids)

    def change_id(self, new_id_for_id):
        """ Changes the id of the specified motors (each id must be unique on the bus). """
        if len(set(new_id_for_id.values())) < len(new_id_for_id):
            raise ValueError('each id must be unique.')

        for new_id in new_id_for_id.itervalues():
            if self.ping(new_id):
                raise ValueError('id {} is already used.'.format(new_id))

        self._change_id(new_id_for_id)

        for motor_id, new_id in new_id_for_id.iteritems():
            if motor_id in self._known_models:
                self._known_models[new_id] = self._known_models[motor_id]
                del self._known_models[motor_id]
            if motor_id in self._known_mode:
                self._known_mode[new_id] = self._known_mode[motor_id]
                del self._known_mode[motor_id]

    def change_baudrate(self, baudrate_for_ids):
        """ Changes the baudrate of the specified motors. """
        self._change_baudrate(baudrate_for_ids)

        for motor_id in baudrate_for_ids:
            if motor_id in self._known_models:
                del self._known_models[motor_id]
            if motor_id in self._known_mode:
                del self._known_mode[motor_id]

    def get_status_return_level(self, ids, **kwargs):
        """ Gets the status level for the specified motors. """
        convert = kwargs['convert'] if 'convert' in kwargs else self._convert
        srl = []
        for id in ids:
            try:
                srl.extend(self._get_status_return_level((id, ),
                           error_handler=None, convert=convert))
            except DxlTimeoutError as e:
                if self.ping(id):
                    srl.append('never' if convert else 0)
                else:
                    if self._error_handler:
                        self._error_handler.handle_timeout(e)
                        return ()
                    else:
                        raise e

        return tuple(srl)

    def set_status_return_level(self, srl_for_id, **kwargs):
        """ Sets status return level to the specified motors. """
        convert = kwargs['convert'] if 'convert' in kwargs else self._convert
        if convert:
            srl_for_id = dict(zip(srl_for_id.keys(),
                              [('never', 'read', 'always').index(s) for s in srl_for_id.values()]))
        self._set_status_return_level(srl_for_id, convert=False)

    def get_mode(self, ids):
        """ Gets the mode ('joint' or 'wheel') for the specified motors. """
        to_get_ids = [id for id in ids if id not in self._known_mode]
        limits = self.get_angle_limit(to_get_ids, convert=False)
        modes = ('wheel' if limit == (0, 0) else 'joint' for limit in limits)

        self._known_mode.update(zip(to_get_ids, modes))

        return tuple(self._known_mode[id] for id in ids)

    def set_wheel_mode(self, ids):
        """ Sets the specified motors to wheel mode. """
        self._set_mode(dict(zip(ids, itertools.repeat('wheel'))))

    def set_joint_mode(self, ids):
        """ Sets the specified motors to joint mode. """
        self._set_mode(dict(zip(ids, itertools.repeat('joint'))))

    def _set_mode(self, mode_for_id):
        models = ['MX' if m.startswith('MX') else '*' for m in self.get_model(list(mode_for_id.keys()))]
        pos_max = [position_range[m][0] for m in models]
        limits = ((0, 0) if mode == 'wheel' else (0, pos_max[i] - 1)
                  for i, mode in enumerate(mode_for_id.itervalues()))

        self._set_angle_limit(dict(zip(mode_for_id.keys(), limits)), convert=False)
        self._known_mode.update(mode_for_id.items())

    def set_angle_limit(self, limit_for_id, **kwargs):
        """ Sets the angle limit to the specified motors. """
        convert = kwargs['convert'] if 'convert' in kwargs else self._convert

        if 'wheel' in self.get_mode(limit_for_id.keys()):
            raise ValueError('can not change the angle limit of a motor in wheel mode')

        if (0, 0) in limit_for_id.values():
            raise ValueError('can not set limit to (0, 0)')

        self._set_angle_limit(limit_for_id, convert=convert)

    def switch_led_on(self, ids):
        """ Switches on the LED of the motors with the specified ids. """
        self._set_LED(dict(zip(ids, itertools.repeat(True))))

    def switch_led_off(self, ids):
        """ Switches off the LED of the motors with the specified ids. """
        self._set_LED(dict(zip(ids, itertools.repeat(False))))

    def enable_torque(self, ids):
        """ Enables torque of the motors with the specified ids. """
        self._set_torque_enable(dict(zip(ids, itertools.repeat(True))))

    def disable_torque(self, ids):
        """ Disables torque of the motors with the specified ids. """
        self._set_torque_enable(dict(zip(ids, itertools.repeat(False))))

    def get_pid_gain(self, ids, **kwargs):
        """ Gets the pid gain for the specified motors. """
        return tuple([tuple(reversed(t)) for t in self._get_pid_gain(ids, **kwargs)])

    def set_pid_gain(self, pid_for_id, **kwargs):
        """ Sets the pid gain to the specified motors. """
        pid_for_id = dict(itertools.izip(pid_for_id.iterkeys(),
                                         [tuple(reversed(t)) for t in pid_for_id.values()]))
        self._set_pid_gain(pid_for_id, **kwargs)

    # MARK: - Generic Getter / Setter

    def get_control_table(self, ids, **kwargs):
        """ Gets the full control table for the specified motors.

            ..note:: This function requires the model for each motor to be known. Querring this additional information might add some extra delay.

            """
        error_handler = kwargs['error_handler'] if ('error_handler' in kwargs) else self._error_handler
        convert = kwargs['convert'] if ('convert' in kwargs) else self._convert

        bl = ('goal position speed load', 'present position speed load')
        controls = [c for c in self._DxlIO__controls if c.name not in bl]

        res = []

        for id, model in zip(ids, self.get_model(ids)):
            controls = [c for c in controls if model in c.models]

            controls = sorted(controls, key=lambda c: c.address)

            address = controls[0].address
            length = controls[-1].address + controls[-1].nb_elem * controls[-1].length

            rp = DxlReadDataPacket(id, address, length)
            sp = self._send_packet(rp, error_handler=error_handler)

            d = OrderedDict()
            for c in controls:
                v = dxl_decode_all(sp.parameters[c.address:c.address+c.nb_elem*c.length], c.nb_elem)
                d[c.name] = c.dxl_to_si(v, model) if convert else v

            res.append(d)

        return tuple(res)

    @classmethod
    def _generate_accessors(cls, control):
        cls.__controls.append(control)

        if control.access in (_DxlAccess.readonly, _DxlAccess.readwrite):
            def my_getter(self, ids, **kwargs):
                return self._get_control_value(control, ids, **kwargs)

            func_name = control.getter_name if control.getter_name else 'get_{}'.format(control.name.replace(' ', '_'))
            func_name = '_{}'.format(func_name) if hasattr(cls, func_name) else func_name
            my_getter.__doc__ = 'Gets {} from the specified motors.'.format(control.name)
            my_getter.__name__ = func_name
            setattr(cls, func_name, my_getter)

        if control.access in (_DxlAccess.writeonly, _DxlAccess.readwrite):
            def my_setter(self, value_for_id, **kwargs):
                self._set_control_value(control, value_for_id, **kwargs)

            func_name = control.setter_name if control.setter_name else 'set_{}'.format(control.name.replace(' ', '_'))
            func_name = '_{}'.format(func_name) if hasattr(cls, func_name) else func_name
            my_setter.__doc__ = 'Sets {} to the specified motors.'.format(control.name)
            my_setter.__name__ = func_name
            setattr(cls, func_name, my_setter)

    def _get_control_value(self, control, ids, **kwargs):
        if not ids:
            return ()

        error_handler = kwargs['error_handler'] if ('error_handler' in kwargs) else self._error_handler
        convert = kwargs['convert'] if ('convert' in kwargs) else self._convert

        if self._sync_read and len(ids) > 1:
            rp = DxlSyncReadPacket(ids, control.address, control.length * control.nb_elem)
            sp = self._send_packet(rp, error_handler=error_handler)

            if not sp:
                return ()

            values = sp.parameters

        else:
            values = []
            for motor_id in ids:
                rp = DxlReadDataPacket(motor_id, control.address, control.length * control.nb_elem)
                sp = self._send_packet(rp, error_handler=error_handler)

                if not sp:
                    return ()

                values.extend(sp.parameters)

        values = list(itertools.izip(*([iter(values)] * control.length * control.nb_elem)))
        values = [dxl_decode_all(value, control.nb_elem) for value in values]

        if not values:
            return ()

        # when using SYNC_READ instead of getting a timeout
        # a non existing motor will "return" the maximum value
        if self._sync_read:
            max_val = 2 ** (8 * control.length) - 1
            if max_val in (itertools.chain(*values) if control.nb_elem > 1 else values):
                lost_ids = []
                for i, v in enumerate(itertools.chain(*values) if control.nb_elem > 1 else values):
                    if v == max_val:
                        lost_ids.append(ids[i // control.nb_elem])
                e = DxlTimeoutError(self, rp, list(set(lost_ids)))
                if self._error_handler:
                    self._error_handler.handle_timeout(e)
                    return ()
                else:
                    raise e

        if convert:
            models = self.get_model(ids)
            if not models:
                return ()
            values = [control.dxl_to_si(v, m) for v, m in zip(values, models)]

        return tuple(values)

    def _set_control_value(self, control, value_for_id, **kwargs):
        if not value_for_id:
            return

        convert = kwargs['convert'] if ('convert' in kwargs) else self._convert

        if convert:
            models = self.get_model(value_for_id.keys())
            if not models:
                return

            value_for_id = dict(zip(value_for_id.keys(),
                                    map(control.si_to_dxl, value_for_id.values(), models)))

        data = []
        for motor_id, value in value_for_id.iteritems():
            data.extend(itertools.chain((motor_id, ),
                                        dxl_code_all(value, control.length, control.nb_elem)))

        wp = DxlSyncWritePacket(control.address, control.length * control.nb_elem, data)
        self._send_packet(wp, wait_for_status_packet=False)

    # MARK: - Send/Receive packet
    def __real_send(self, instruction_packet, wait_for_status_packet, _force_lock):
        if self.closed:
            raise DxlError('try to send a packet on a closed serial communication')

        logger.debug('Sending %s', instruction_packet,
                     extra={'port': self.port,
                            'baudrate': self.baudrate,
                            'timeout': self.timeout})

        with self.__force_lock(_force_lock) or self._serial_lock:
            self.flush(_force_lock=True)

            data = instruction_packet.to_string()
            nbytes = self._serial.write(data)
            if len(data) != nbytes:
                raise DxlCommunicationError(self,
                                            'instruction packet not entirely sent',
                                            instruction_packet)

            if not wait_for_status_packet:
                return

            data = self._serial.read(DxlPacketHeader.length)
            if not data:
                raise DxlTimeoutError(self, instruction_packet, instruction_packet.id)

            try:
                header = DxlPacketHeader.from_string(data)
                data += self._serial.read(header.packet_length)
                status_packet = DxlStatusPacket.from_string(data)

            except ValueError:
                msg = 'could not parse received data {}'.format(bytearray(data))
                raise DxlCommunicationError(self, msg, instruction_packet)

            logger.debug('Receiving %s', status_packet,
                         extra={'port': self.port,
                                'baudrate': self.baudrate,
                                'timeout': self.timeout})

            return status_packet

    def _send_packet(self,
                     instruction_packet, wait_for_status_packet=True,
                     error_handler=None,
                     _force_lock=False):
        if not error_handler:
            return self.__real_send(instruction_packet, wait_for_status_packet, _force_lock)

        try:
            sp = self.__real_send(instruction_packet, wait_for_status_packet, _force_lock)

            if sp and sp.error:
                errors = decode_error(sp.error)
                for e in errors:
                    handler_name = 'handle_{}'.format(e.lower().replace(' ', '_'))
                    f = operator.methodcaller(handler_name, instruction_packet)
                    f(error_handler)

            return sp

        except DxlTimeoutError as e:
            error_handler.handle_timeout(e)

        except DxlCommunicationError as e:
            error_handler.handle_communication_error(e)


# MARK: - Dxl Errors
class DxlError(Exception):
    """ Base class for all errors encountered using :class:`~pypot.dynamixel.io.DxlIO`. """
    pass


class DxlCommunicationError(DxlError):
    """ Base error for communication error encountered when using :class:`~pypot.dynamixel.io.DxlIO`. """
    def __init__(self, dxl_io, message, instruction_packet):
        self.dxl_io = dxl_io
        self.message = message
        self.instruction_packet = instruction_packet

    def __str__(self):
        return '{self.message} after sending {self.instruction_packet}'.format(self=self)


class DxlTimeoutError(DxlCommunicationError):
    """ Timeout error encountered when using :class:`~pypot.dynamixel.io.DxlIO`. """
    def __init__(self, dxl_io, instruction_packet, ids):
        DxlCommunicationError.__init__(self, dxl_io, 'timeout occured', instruction_packet)
        self.ids = ids

    def __str__(self):
        return 'motors {} did not respond after sending {}'.format(self.ids, self.instruction_packet)

# MARK: - Generate the accessors


def _add_control(name,
                 address, length=2, nb_elem=1,
                 access=_DxlAccess.readwrite,
                 models=set(dynamixelModels.values()),
                 dxl_to_si=lambda val, model: val,
                 si_to_dxl=lambda val, model: val,
                 getter_name=None,
                 setter_name=None):

    control = _DxlControl(name,
                          address, length, nb_elem,
                          access,
                          models,
                          dxl_to_si, si_to_dxl,
                          getter_name, setter_name)

    DxlIO._generate_accessors(control)


_add_control('model',
             address=0x00,
             access=_DxlAccess.readonly,
             dxl_to_si=dxl_to_model)

_add_control('firmware',
             address=0x02, length=1,
             access=_DxlAccess.readonly)

_add_control('id',
             address=0x03, length=1,
             access=_DxlAccess.writeonly,
             setter_name='change_id')

_add_control('baudrate',
             address=0x04, length=1,
             access=_DxlAccess.writeonly,
             setter_name='change_baudrate',
             si_to_dxl=baudrate_to_dxl)

_add_control('return delay time',
             address=0x05, length=1,
             dxl_to_si=dxl_to_rdt,
             si_to_dxl=rdt_to_dxl)

_add_control('angle limit',
             address=0x06, nb_elem=2,
             dxl_to_si=lambda value, model: (dxl_to_degree(value[0], model),
                                             dxl_to_degree(value[1], model)),
             si_to_dxl=lambda value, model: (degree_to_dxl(value[0], model),
                                             degree_to_dxl(value[1], model)))

_add_control('drive mode',
             address=0x0A, length=1,
             access=_DxlAccess.readwrite,
             models=('MX-106', ),
             dxl_to_si=dxl_to_drive_mode,
             si_to_dxl=drive_mode_to_dxl)

_add_control('highest temperature limit',
             address=0x0B, length=1,
             dxl_to_si=dxl_to_temperature,
             si_to_dxl=temperature_to_dxl)

_add_control('voltage limit',
             address=0x0C, length=1, nb_elem=2,
             dxl_to_si=lambda value, model: (dxl_to_voltage(value[0], model),
                                             dxl_to_voltage(value[1], model)),
             si_to_dxl=lambda value, model: (voltage_to_dxl(value[0], model),
                                             voltage_to_dxl(value[1], model)))

_add_control('max torque',
             address=0x0E,
             dxl_to_si=dxl_to_torque,
             si_to_dxl=torque_to_dxl)

_add_control('status return level',
             address=0x10, length=1,
             dxl_to_si=dxl_to_status,
             si_to_dxl=status_to_dxl)

_add_control('alarm LED',
             address=0x11, length=1,
             dxl_to_si=dxl_to_alarm,
             si_to_dxl=alarm_to_dxl)

_add_control('alarm shutdown',
             address=0x12, length=1,
             dxl_to_si=dxl_to_alarm,
             si_to_dxl=alarm_to_dxl)

_add_control('torque_enable',
             address=0x18, length=1,
             dxl_to_si=dxl_to_bool,
             si_to_dxl=bool_to_dxl,
             getter_name='is_torque_enabled',
             setter_name='_set_torque_enable')

_add_control('LED',
             address=0x19, length=1,
             dxl_to_si=dxl_to_bool,
             si_to_dxl=bool_to_dxl,
             setter_name='_set_LED',
             getter_name='is_led_on')

_add_control('pid gain',
             address=0x1A, length=1, nb_elem=3,
             models=('MX-28', 'MX-64', 'MX-106'),
             dxl_to_si=dxl_to_pid,
             si_to_dxl=pid_to_dxl)

_add_control('compliance margin',
             address=0x1A, length=1, nb_elem=2,
             models=('AX-12', 'AX-18', 'RX-28', 'RX-64'))

_add_control('compliance slope',
             address=0x1C, length=1, nb_elem=2,
             models=('AX-12', 'AX-18', 'RX-28', 'RX-64'))

_add_control('goal position',
             address=0x1E,
             dxl_to_si=dxl_to_degree,
             si_to_dxl=degree_to_dxl)

_add_control('moving speed',
             address=0x20,
             dxl_to_si=dxl_to_speed,
             si_to_dxl=speed_to_dxl)

_add_control('torque limit',
             address=0x22,
             dxl_to_si=dxl_to_torque,
             si_to_dxl=torque_to_dxl)

_add_control('goal position speed load',
             address=0x1E, nb_elem=3,
             dxl_to_si=lambda value, model: (dxl_to_degree(value[0], model),
                                             dxl_to_speed(value[1], model),
                                             dxl_to_load(value[2], model)),
             si_to_dxl=lambda value, model: (degree_to_dxl(value[0], model),
                                             speed_to_dxl(value[1], model),
                                             torque_to_dxl(value[2], model)))

_add_control('present position',
             address=0x24,
             access=_DxlAccess.readonly,
             dxl_to_si=dxl_to_degree)

_add_control('present speed',
             address=0x26,
             access=_DxlAccess.readonly,
             dxl_to_si=dxl_to_speed)

_add_control('present load',
             address=0x28,
             access=_DxlAccess.readonly,
             dxl_to_si=dxl_to_load)

_add_control('present position speed load',
             address=0x24, nb_elem=3,
             access=_DxlAccess.readonly,
             dxl_to_si=lambda value, model: (dxl_to_degree(value[0], model),
                                             dxl_to_speed(value[1], model),
                                             dxl_to_load(value[2], model)))

_add_control('present voltage',
             address=0x2A, length=1,
             access=_DxlAccess.readonly,
             dxl_to_si=dxl_to_voltage)

_add_control('present temperature',
             address=0x2B, length=1,
             access=_DxlAccess.readonly,
             dxl_to_si=dxl_to_temperature)

_add_control('moving',
             address=0x2E, length=1,
             access=_DxlAccess.readonly,
             dxl_to_si=dxl_to_bool,
             getter_name='is_moving')



import os
import json
import argparse

from subprocess import call, check_output

from pypot.robot import from_json

N = 50000
T = 15 * 60


if __name__ == '__main__':
    parser = argparse.ArgumentParser()

    parser.add_argument('-l', '--log-folder',
                        type=str, required=True)

    parser.add_argument('-c', '--config',
                        type=str, required=True)

    args = parser.parse_args()

    bp = args.log_folder
    if os.path.exists(bp):
        raise IOError('Folder already exists: {}'.format(bp))
    os.mkdir(bp)

    call(["python", "dxl-single-motor.py",
          "-l", os.path.join(bp, 'dxl_single'),
          "-N", str(N)])

    call(["python", "dxl-controller.py",
          "-l", os.path.join(bp, 'dxl_controller.list'),
          "-N", str(N),
          "-c", args.config])

    call(["python", "robot.py",
          "-l", os.path.join(bp, 'robot.list'),
          "-t", str(T),
          "-c", args.config])

    print('Everything is done!!!')



import os
import time
import numpy
import argparse
import itertools

from pypot.dynamixel import DxlIO, get_available_ports
from pypot.dynamixel.packet import DxlReadDataPacket, DxlSyncWritePacket
from pypot.dynamixel.conversion import dxl_code


def timeit(rw_func, N, destfile):
    dt = []
    for _ in range(N):
        start = time.time()
        rw_func()
        end = time.time()

        dt.append(end - start)

    # We'll use raw file instead of numpy because of pypy
    with open(destfile, 'w') as f:
        f.write(str(dt))

    return numpy.mean(dt)

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('-l', '--log-folder',
                        type=str, required=True)
    parser.add_argument('-N', type=int, required=True)
    args = parser.parse_args()

    bp = args.log_folder
    if os.path.exists(bp):
        raise IOError('Folder already exists: {}'.format(bp))
    os.mkdir(bp)

    available_ports = get_available_ports()

    port = available_ports[0]
    print('Connecting on port {}.'.format(port))
    dxl = DxlIO(port)

    print('Scanning motors on the bus...')
    ids = dxl.scan()
    print('Found {}'.format(ids))

    ids = ids[:1]
    print('Will use id: {}'.format(ids))

    if dxl.get_return_delay_time(ids)[0] != 0:
        raise IOError('Make sure your motor has return delay time set to 0')

    print('Start testing !')

    print('Using "normal" pypot package...')
    def rw_pypot():
        dxl.get_present_position(ids)
        dxl.set_goal_position(dict(zip(ids, itertools.repeat(0))))

    dt = timeit(rw_pypot, args.N, os.path.join(bp, 'rw_pypot.list'))
    print('in {}ms'.format(dt * 1000))

    print('Using pref-forged packet...')
    c_get = [c for c in DxlIO._DxlIO__controls
             if c.name == 'present position'][0]

    c_set = [c for c in DxlIO._DxlIO__controls
             if c.name == 'goal position'][0]

    pos = dxl_code(0, c_set.length)
    rp = DxlReadDataPacket(ids[0], c_get.address, c_get.length)
    sp = DxlSyncWritePacket(c_set.address, c_set.length, ids[:1] + list(pos))

    def rw_forged_packet():
        dxl._send_packet(rp)
        dxl._send_packet(sp, wait_for_status_packet=False)

    dt = timeit(rw_forged_packet, args.N, os.path.join(bp, 'rw_forged.list'))
    print('in {}ms'.format(dt * 1000))

    print('Using raw serial communication...')
    s_read = rp.to_string()
    s_write = sp.to_string()

    def rw_serial():
        dxl._serial.write(s_read)
        dxl._serial.read(8)
        dxl._serial.write(s_write)
    dt = timeit(rw_serial, args.N, os.path.join(bp, 'rw_serial.list'))
    print('in {}ms'.format(dt * 1000))

    print('Done !')



import os
import time
import psutil
import argparse

from numpy import mean

from pypot.robot import from_json


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('-l', '--log-file',
                        type=str, required=True)
    parser.add_argument('-t', '--running-time',
                        type=int, required=True)
    parser.add_argument('-c', '--config',
                        type=str, required=True)
    args = parser.parse_args()

    bp = args.log_file
    if os.path.exists(bp):
        raise IOError('File already exists: {}'.format(bp))

    p = psutil.Process()
    p.cpu_percent()

    robot = from_json(args.config)
    robot.start_sync()
    time.sleep(60)

    loads = []
    start = time.time()
    while time.time() - start < args.running_time:
        loads.append(p.cpu_percent())
        time.sleep(1)
    load = mean(loads)
    print('After loading about {}%'.format(load))

    # We'll use raw file instead of numpy because of pypy
    with open(bp, 'w') as f:
        f.write(str(loads))



import numpy

from .primitive import LoopPrimitive


class Sinus(LoopPrimitive):
    """ Apply a sinus on the motor specified as argument. Parameters (amp, offset and phase) should be specified in degree
    """
    def __init__(self, robot, refresh_freq,
                 motor_list,
                 amp=1, freq=0.5, offset=0, phase=0):

        LoopPrimitive.__init__(self, robot, refresh_freq)

        self._freq = freq
        self._amp = amp
        self._offset = offset
        self._phase = phase

        self.motor_list = [self.get_mockup_motor(m) for m in motor_list]

    def update(self):
        """ Compute the sin(t) where t is the elapsed time since the primitive has been started. """
        pos = self._amp * numpy.sin(self._freq * 2.0 * numpy.pi * self.elapsed_time +
                              self._phase * numpy.pi / 180.0) + self._offset

        for m in self.motor_list:
            m.goal_position = pos

    @property
    def frequency(self):
        return self._freq

    @frequency.setter
    def frequency(self, new_freq):
        self._freq = new_freq

    @property
    def amplitude(self):
        return self._amp

    @amplitude.setter
    def amplitude(self, new_amp):
        self._amp = new_amp

    @property
    def offset(self):
        return self._offset

    @offset.setter
    def offset(self, new_offset):
        self._offset = new_offset

    @property
    def phase(self):
        return self._phase

    @phase.setter
    def phase(self, new_phase):
        self._phase = new_phase


class Cosinus(Sinus):
    """ Apply a cosinus on the motor specified as argument. Parameters (amp, offset and phase) should be specified in degree
    """
    def __init__(self, robot, refresh_freq,
                 motor_list,
                 amp=1, freq=0.5, offset=0, phase=0):

        Sinus.__init__(self, robot, refresh_freq,
                       motor_list,
                       amp, freq, offset, phase=(numpy.pi / 2 + phase))



import threading
import time


class DxlController(object):
    def __init__(self, dxl_io, dxl_motors):
        """ Synchronizes the reading/writing of :class:`~pypot.dynamixel.motor.DxlMotor` with the real motors.

            This class handles synchronization loops that automatically read/write values from the "software" :class:`~pypot.dynamixel.motor.DxlMotor` with their "hardware" equivalent. Those loops shared a same :class:`~pypot.dynamixel.io.DxlIO` connection to avoid collision in the bus. Each loop run within its own thread at its own frequency.

            .. warning:: As all the loop attached to a controller shared the same bus, you should make sure that they can run without slowing down the other ones.

            """
        self._dxl_io = dxl_io

        self._motors = dxl_motors
        self._ids = tuple(m.id for m in self._motors)

        self._loops = []

    def close(self):
        """ Stops the synchronization loop and closes the dynamixel connection. """
        self.stop()
        self._dxl_io.close()

    def start(self):
        """ Starts all the synchronization loops. """
        for l in self._loops:
            l.start()
            l._started.wait()

    def stop(self):
        """ Stops al the synchronization loops (they can not be started again). """
        loops_to_stop = filter(lambda l: l.is_alive(), self._loops)
        [l.stop() for l in loops_to_stop]
        [l.join() for l in loops_to_stop]

    def add_sync_loop(self, freq, function, name):
        """ Adds a synchronization loop that will run a function at a predefined freq. """
        sl = _RepeatedTimer(freq, function, name)
        self._loops.append(sl)

    def add_read_loop(self, freq, regname, varname=None):
        """ Adds a read loop that will get the value from the specified register and set it to the :class:`~pypot.dynamixel.motor.DxlMotor`. """
        varname = varname if varname else regname
        self.add_sync_loop(freq, lambda: self._get_register(regname, varname),
                           'Thread-get_{}'.format(regname))

    def _get_register(self, regname, varname):
        motors = [m for m in self._motors if hasattr(m, varname)]
        if not motors:
            return
        ids = [m.id for m in motors]

        values = getattr(self._dxl_io, 'get_{}'.format(regname))(ids)
        for m, val in zip(motors, values):
            m._values[varname] = val

    def add_write_loop(self, freq, regname, varname=None):
        """ Adds a write loop that will get the value from :class:`~pypot.dynamixel.motor.DxlMotor` and set it to the specified register. """
        varname = varname if varname else regname

        # We force a get to initalize the synced var to their current values
        self._get_register(regname, varname)

        self.add_sync_loop(freq, lambda: self._set_register(regname, varname),
                           'Thread-set_{}'.format(regname))

    def _set_register(self, regname, varname):
        motors = [m for m in self._motors if hasattr(m, varname)]

        if not motors:
            return
        ids = [m.id for m in motors]

        values = (m._values[varname] for m in motors)
        getattr(self._dxl_io, 'set_{}'.format(regname))(dict(zip(ids, values)))


class BaseDxlController(DxlController):
    """ Implements a basic controller that synchronized the most frequently used values.

        More precisely, this controller:
            * reads the present position, speed, load at 50Hz
            * writes the goal position, moving speed and torque limit at 50Hz
            * writes the pid gains (or compliance margin and slope) at 10Hz
            * reads the present voltage and temperature at 1Hz

        """
    def __init__(self, dxl_io, dxl_motors):
        DxlController.__init__(self, dxl_io, dxl_motors)

        self.add_sync_loop(50, self._get_pos_speed_load, 'Thread-get_pos_speed_load')
        self.add_sync_loop(50, self._set_pos_speed_load, 'Thread-set_pos_speed_load')

        self.add_write_loop(10, 'pid_gain', 'pid')
        self.add_write_loop(10, 'compliance_margin')
        self.add_write_loop(10, 'compliance_slope')

        self.add_read_loop(1, 'angle_limit')
        self.add_read_loop(1, 'present_voltage')
        self.add_read_loop(1, 'present_temperature')

        torques = self._dxl_io.is_torque_enabled(self._ids)
        for m, c in zip(self._motors, torques):
            m.compliant = not c
        self._old_torques = torques

        values = self._dxl_io.get_goal_position_speed_load(self._ids)
        positions, speeds, loads = zip(*values)
        for m, p, s, l in zip(self._motors, positions, speeds, loads):
            m._values['goal_position'] = p
            m._values['moving_speed'] = s
            m._values['torque_limit'] = l

    def _get_pos_speed_load(self):
        values = self._dxl_io.get_present_position_speed_load(self._ids)

        if not values:
            return

        positions, speeds, loads = zip(*values)

        for m, p, s, l in zip(self._motors, positions, speeds, loads):
            m._values['present_position'] = p
            m._values['present_speed'] = s
            m._values['present_load'] = l

    def _set_pos_speed_load(self):
        change_torque = {}
        torques = [not m.compliant for m in self._motors]
        for m, t, old_t in zip(self._motors, torques, self._old_torques):
            if t != old_t:
                change_torque[m.id] = t
        self._old_torques = torques
        if change_torque:
            self._dxl_io._set_torque_enable(change_torque)

        rigid_motors = [m for m in self._motors if not m.compliant]
        ids = tuple(m.id for m in rigid_motors)

        if not ids:
            return

        values = ((m._values['goal_position'],
                   m._values['moving_speed'],
                   m._values['torque_limit']) for m in rigid_motors)
        self._dxl_io.set_goal_position_speed_load(dict(zip(ids, values)))


class _RepeatedTimer(threading.Thread):
    def __init__(self, freq, function, name=None):
        threading.Thread.__init__(self, name=name)
        self.daemon = True

        self.period = 1.0 / freq
        self.function = function

        self._running = threading.Event()
        self._running.set()

        self._started = threading.Event()

    def run(self):
        while self._running.is_set():
            start = time.time()

            self.function()

            if not self._started.is_set():
                self._started.set()

            end = time.time()

            st = self.period - (end - start)
            if st > 0:
                time.sleep(st)

    def stop(self):
        self._running.clear()



import argparse
import numpy
import time
import sys

import pypot.dynamixel

N = 1000


def read_register(dxl, register, ids):
    print '\tReading {} times the {}...'.format(N, register)

    t = []
    getter = getattr(dxl, 'get_{}'.format(register))

    for _ in range(N):
        start = time.time()
        getter(ids)
        end = time.time()
        t.append(1000 * (end - start))

    print '\tTook {}ms (STD={}) per read'.format(numpy.mean(t), numpy.std(t))
    print


def full_test(dxl, ids):
    print 'Testing the communication speed with motor{} {}'.format('s' if len(ids) else '',
                                                                   ids)
    read_register(dxl, 'present_position', ids)
    read_register(dxl, 'control_table', ids)


if __name__ == '__main__':
    available_ports = pypot.dynamixel.get_available_ports()

    parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter,
                                     description='Test low-level communication with Dynamixel motors.')

    parser.add_argument('-p', '--ports',
                        type=str, nargs='+',
                        default=available_ports,
                        help='Select which port(s) to test')

    parser.add_argument('-b', '--baudrate',
                        type=int, choices=[57600, 1000000], default=1000000,
                        help='Sets the baudrate')

    args = parser.parse_args()

    for port in args.ports:
        print 'Now testing port {} with {}bds...'.format(port, args.baudrate)

        print 'Opening connection...',
        with pypot.dynamixel.DxlIO(port, baudrate=args.baudrate) as dxl:
            print 'Ok!'
            print 'Scanning the bus...',
            sys.stdout.flush()
            ids = dxl.scan()
            print 'Done!'
            print 'Ids found: {}'.format(ids)

            full_test(dxl, ids[:1])
            full_test(dxl, ids)

        print 'Closing port'
        print


import zmq
import numpy
import threading

from collections import namedtuple


from ...utils import Point3D, Point2D, Quaternion


torso_joints = ('hip_center', 'spine', 'shoulder_center', 'head')
left_arm_joints = ('shoulder_left', 'elbow_left', 'wrist_left', 'hand_left')
right_arm_joints = ('shoulder_right', 'elbow_right', 'wrist_right', 'hand_right')
left_leg_joints = ('hip_left', 'knee_left', 'ankle_left', 'foot_left')
right_leg_joints = ('hip_right', 'knee_right', 'ankle_right', 'foot_right')
skeleton_joints = torso_joints + left_arm_joints + right_arm_joints + left_leg_joints + right_leg_joints


class Skeleton(namedtuple('Skeleton', ('timestamp', 'user_id') + skeleton_joints)):
    joints = skeleton_joints


Joint = namedtuple('Joint', ('position', 'orientation', 'pixel_coordinate'))


class KinectSensor(object):
    def __init__(self, addr, port):
        self._lock = threading.Lock()
        self._skeleton = None

        context = zmq.Context()
        self.socket = context.socket(zmq.SUB)
        self.socket.connect('tcp://{}:{}'.format(addr, port))
        self.socket.setsockopt(zmq.SUBSCRIBE, '')

        t = threading.Thread(target=self.get_skeleton)
        t.daemon = True
        t.start()

    @property
    def tracked_skeleton(self):
        with self._lock:
            return self._skeleton

    @tracked_skeleton.setter
    def tracked_skeleton(self, skeleton):
        with self._lock:
            self._skeleton = skeleton

    def get_skeleton(self):
        while True:
            md = self.socket.recv_json()
            msg = self.socket.recv()

            skeleton_array = numpy.frombuffer(buffer(msg), dtype=md['dtype'])
            skeleton_array = skeleton_array.reshape(md['shape'])

            joints = []
            for i in range(len(skeleton_joints)):
                x, y, z, w = skeleton_array[i][0:4]
                position = Point3D(x / w, y / w, z / w)
                pixel_coord = Point2D(*skeleton_array[i][4:6])
                orientation = Quaternion(*skeleton_array[i][6:10])
                joints.append(Joint(position, orientation, pixel_coord))

            self.tracked_skeleton = Skeleton(md['timestamp'], md['user_index'], *joints)


if __name__ == '__main__':
    import cv2

    kinect = KinectSensor('193.50.110.60', 9999)

    while True:
        img = numpy.zeros((480, 640, 3))

        skeleton = kinect.tracked_skeleton

        if skeleton:
            for joint_name in skeleton.joints:
                x, y = getattr(skeleton, joint_name).pixel_coordinate
                pt = (int(x * 640), int(y * 480))
                cv2.circle(img, pt, 5, (255, 255, 255), thickness=-1)

        cv2.imshow('Skeleton', img)
        cv2.waitKey(50)



import json

from ..primitive import Primitive
from .request import BaseRequestHandler


# TODO: The server should spawn a new primitive for each client


class AbstractServer(Primitive):
    """ Abstract Server which mostly delegate the work to a request handler. """
    def __init__(self, robot, handler=BaseRequestHandler):
        Primitive.__init__(self, robot)

        self.request_handler = BaseRequestHandler(self.robot)


class MyJSONEncoder(json.JSONEncoder):
    """ JSONEncoder which tries to call a json property before using the enconding default function. """
    def default(self, obj):
        return obj.json if hasattr(obj, 'json') else json.JSONEncoder.default(self, obj)



import numpy
import Queue
import time
import zmq

from pykinect import nui
from pykinect.nui import JointId


class KinectServer(object):
    def __init__(self, addr, port):
        context = zmq.Context()

        self.socket = context.socket(zmq.PUB)
        self.socket.bind('tcp://{}:{}'.format(addr, port))

        self.msg = Queue.Queue(2)

        self.kinect = nui.Runtime()

        self.kinect.skeleton_frame_ready += self.skeleton_frame_ready
        self.kinect.skeleton_engine.enabled = True

    def serve_forever(self):
        print 'Server started, waiting for connection...'

        while True:
            msg = self.msg.get()
            self.send_skeleton(msg[0], msg[1])

    def send_skeleton(self, user_index, skeleton_array, flags=0, copy=True, track=False):
        md = dict(
            user_index=user_index,
            timestamp=time.time(),
            dtype=str(skeleton_array.dtype),
            shape=skeleton_array.shape,
            )

        self.socket.send_json(md, flags | zmq.SNDMORE)
        return self.socket.send(skeleton_array, flags, copy=copy, track=track)

    def skeleton_frame_ready(self, frame):
        self.msg.queue.clear()

        for data in frame.SkeletonData:
            pos = numpy.zeros((JointId.Count, 6))

            for i in range(JointId.Count):
                p = data.SkeletonPositions[i]
                pos[i][0:4] = (p[0], p[1], p[2], p[3])
                pos[i][4:6] = nui.SkeletonEngine.skeleton_to_depth_image(p)

            rot_data = data.calculate_bone_orientations()

            rot = numpy.zeros((JointId.Count, 4))
            for i in range(JointId.Count):
                q = rot_data[i].hierarchical_rotation.rotation_quaternion
                rot[i] = [q.x, q.y, q.z, q.w]

            skeleton_array = numpy.concatenate((pos.T, rot.T)).T.copy()

            if not (numpy.all(skeleton_array[:, 0:9] == 0) and
                    numpy.all(skeleton_array[:, 9] == 1)):
                self.msg.put(('{}'.format(data.dwUserIndex), skeleton_array.copy()))


if __name__ == '__main__':
    import socket
    host = socket.gethostbyname(socket.gethostname())

    kinect_server = KinectServer(host, 9999)
    kinect_server.serve_forever()



import json

from .primitive import LoopPrimitive


class Move(object):
    """ Simple class used to represent a movement.

    This class simply wraps a sequence of positions of specified motors. The sequence must be recorded at a predefined frequency. This move can be recorded through the :class:`~pypot.primitive.move.MoveRecorder` class and played thanks to a :class:`~pypot.primitive.move.MovePlayer`.

    """
    def __init__(self, freq):
        self._framerate = freq
        self._positions = []

    def __repr__(self):
        return '<Move framerate={} #keyframes={}>'.format(self.framerate,
                                                          len(self.positions()))

    def __getitem__(self, i):
        return self._positions[i]

    @property
    def framerate(self):
        return self._framerate

    def add_position(self, pos):
        """ Add a new position to the movement sequence.

        Each position is typically stored as a dict of (motor_name, motor_position).
        """
        self._positions.append(pos)

    def iterpositions(self):
        """ Returns an iterator on the stored positions. """
        return iter(self._positions)

    def positions(self):
        """ Returns a copy of the stored positions. """
        return list(self.iterpositions())

    def save(self, file):
        """ Saves the :class:`~pypot.primitive.move.Move` to a json file.

        .. note:: The format used to store the :class:`~pypot.primitive.move.Move` is extremely verbose and should be obviously optimized for long moves.
        """
        d = {
            'framerate': self.framerate,
            'positions': self.positions(),
        }
        json.dump(d, file, indent=2)

    @classmethod
    def load(cls, file):
        """ Loads a :class:`~pypot.primitive.move.Move` from a json file. """
        d = json.load(file)

        move = cls(d['framerate'])
        move._positions = d['positions']
        return move


class MoveRecorder(LoopPrimitive):
    """ Primitive used to record a :class:`~pypot.primitive.move.Move`.

    The recording can be :meth:`~pypot.primitive.primitive.Primitive.start` and :meth:`~pypot.primitive.primitive.Primitive.stop` by using the :class:`~pypot.primitive.primitive.LoopPrimitive` methods.

    .. note:: Re-starting the recording will create a new :class:`~pypot.primitive.move.Move` losing all the previously stored data.

    """
    def __init__(self, robot, freq, tracked_motors):
        LoopPrimitive.__init__(self, robot, freq)
        self.freq = freq

        self.tracked_motors = map(self.get_mockup_motor, tracked_motors)

    def setup(self):
        self._move = Move(self.freq)

    def update(self):
        position = dict([(m.name, m.present_position) for m in self.tracked_motors])
        self._move.add_position(position)

    @property
    def move(self):
        """ Returns the currently recorded :class:`~pypot.primitive.move.Move`. """
        return self._move


class MovePlayer(LoopPrimitive):
    """ Primitive used to play a :class:`~pypot.primitive.move.Move`.

    The playing can be :meth:`~pypot.primitive.primitive.Primitive.start` and :meth:`~pypot.primitive.primitive.Primitive.stop` by using the :class:`~pypot.primitive.primitive.LoopPrimitive` methods.

    .. warning:: You should be careful that you primitive actually runs at the same speed that the move has been recorded. If the player can not run as fast as the framerate of the :class:`~pypot.primitive.move.Move`, it will be played slowly resulting in a slower version of your move.
    """

    def __init__(self, robot, move):
        LoopPrimitive.__init__(self, robot, move.framerate)
        self.move = move

    def setup(self):
        self.positions = self.move.iterpositions()

    def update(self):
        try:
            position = self.positions.next()

            for m, v in position.iteritems():
                getattr(self.robot, m).goal_position = v

        except StopIteration:
            self.stop()



import bottle
import logging
import json

from .server import AbstractServer, MyJSONEncoder


logger = logging.getLogger(__name__)


class HTTPServer(AbstractServer):
    """ Bottle based HTTPServer used to remote access a robot.

        The server answers to the following requests:

        * GET /motor/list.json
        * GET /primitive/list.json
        * GET /motor/<name>/register.json (or GET /<name>/register.json)
        * GET /motor/<name>/<register> (or GET /<name>/<register>)
        * POST /motor/<name>/<register> (or POST /<name>/<register>)
        * POST /primitive/<prim_name>/call/<meth_name> (or GET /<prim_name>/call/<meth_name>)
        * POST /request.json

     """
    def __init__(self, robot, host='localhost', port=8080):
        AbstractServer.__init__(self, robot)

        self.host = host
        self.port = port

        self.app = bottle.Bottle()
        self.app.install(bottle.JSONPlugin(json_dumps=lambda s: json.dumps(s, cls=MyJSONEncoder)))

        @self.app.get('/motor/list.json')
        def get_motor_list():
            req = {
                'get': {
                    '': ['motors', ]
                }
            }

            return self.request_handler.handle_request(req)['get']['']

        @self.app.get('/primitive/list.json')
        def get_primitive_list():
            req = {
                'get': {
                    '': ['attached_primitives_name', ]
                }
            }
            return self.request_handler.handle_request(req)['get']['']

        @self.app.get('/<name>/register.json')
        @self.app.get('/motor/<name>/register.json')
        def get_motor_register(name):
            req = {
                'get': {
                    name: ['registers', ]
                }
            }
            return self.request_handler.handle_request(req)['get'][name]

        @self.app.get('/<name>/<register>')
        @self.app.get('/motor/<name>/<register>')
        def get_object_register(name, register):
            req = {
                'get': {
                    name: [register, ]
                }
            }

            return self.request_handler.handle_request(req)['get'][name]

        @self.app.post('/<name>/<register>')
        @self.app.post('/motor/<name>/<register>')
        def set_object_register(name, register):
            req = {
                'set': {
                    name: {register: bottle.request.json}
                }
            }

            return self.request_handler.handle_request(req)

        @self.app.post('/<prim_name>/call/<meth_name>')
        @self.app.post('/primitive/<prim_name>/call/<meth_name>')
        def call_prim_meth(prim_name, meth_name):
            req = {
                'call': {
                    prim_name: {meth_name: bottle.request.json}
                }
            }
            return self.request_handler.handle_request(req)['call'][prim_name]

        @self.app.post('/request.json')
        def request():
            return self.request_handler.handle_request(bottle.request.json)

        logger.info('Starting HTTPServer on http://%s:%s', host, port)

    def run(self):
        """ Start the bottle server, run forever. """
        bottle.run(self.app,
                   host=self.host, port=self.port,
                   quiet=True,
                   server='tornado')



# -*- coding: utf-8 -*-

from .io import logger


class DxlErrorHandler(object):
    """ This class is used to represent all the error that you can/should handle.

        The errors can be of two types:

        * communication error (timeout, communication)
        * motor error (voltage, limit, overload...)

        This class was designed as an abstract class and so you should write your own handler by subclassing this class and defining the apropriate behavior for your program.

        .. warning:: The motor error should be overload carrefuly as they can indicate important mechanical issue.

        """
    # MARK: - Communication errors

    def handle_timeout(self, timeout_error):
        raise NotImplementedError

    def handle_communication_error(self, communication_error):
        raise NotImplementedError

    # MARK: - Motor errors
    def handle_input_voltage_error(self, instruction_packet):
        raise NotImplementedError

    def handle_angle_limit_error(self, instruction_packet):
        raise NotImplementedError

    def handle_overheating_error(self, instruction_packet):
        raise NotImplementedError

    def handle_range_error(self, instruction_packet):
        raise NotImplementedError

    def handle_checksum_error(self, instruction_packet):
        raise NotImplementedError

    def handle_overload_error(self, instruction_packet):
        raise NotImplementedError

    def handle_instruction_error(self, instruction_packet):
        raise NotImplementedError

    def handle_none_error(self, instruction_packet):
        raise NotImplementedError


class BaseErrorHandler(DxlErrorHandler):
    """ This class is a basic handler that just skip the communication errors. """
    def handle_timeout(self, timeout_error):
        msg = 'Timeout after sending {} to motors {}'.format(timeout_error.instruction_packet,
                                                             timeout_error.ids)
        logger.warning(msg,
                       extra={'port': timeout_error.dxl_io.port,
                              'baudrate': timeout_error.dxl_io.baudrate,
                              'timeout': timeout_error.dxl_io.timeout})

    def handle_communication_error(self, com_error):
        msg = 'Communication error after sending {}'.format(com_error.instruction_packet)

        logger.warning(msg,
                       extra={'port': com_error.dxl_io.port,
                              'baudrate': com_error.dxl_io.baudrate,
                              'timeout': com_error.dxl_io.timeout})



# -*- coding: utf-8 -*-


"""
    This module describes all the conversion method used to transform value from the representation used by the dynamixel motor to a more standard form (e.g. degrees, volt...).

    For compatibility issue all comparison method should be written in the following form (even if the model is not actually used):
        * def my_conversion_from_dxl_to_si(value, model): ...
        * def my_conversion_from_si_to_dxl(value, model): ...

    .. note:: If the control is readonly you only need to write the dxl_to_si conversion.

    """

import numpy
import itertools


# MARK: - Position

position_range = {
    'MX': (4096, 360.0),
    '*': (1024, 300.0)
}

def dxl_to_degree(value, model):
    model = 'MX' if model.startswith('MX') else '*'
    max_pos, max_deg = position_range[model]

    return round(((max_deg * float(value)) / (max_pos - 1)) - (max_deg / 2), 2)

def degree_to_dxl(value, model):
    model = 'MX' if model.startswith('MX') else '*'
    max_pos, max_deg = position_range[model]

    pos = int(round((max_pos - 1) * ((max_deg / 2 + float(value)) / max_deg), 0))
    pos = min(max(pos, 0), max_pos - 1)

    return pos

# MARK: - Speed

def dxl_to_speed(value, model):
    cw, speed = divmod(value, 1024)
    direction = (-2 * cw + 1)

    speed_factor = 0.114 if model.startswith('MX') else 0.111

    return direction * (speed * speed_factor) * 6

def speed_to_dxl(value, model):
    direction = 1024 if value < 0 else 0
    speed_factor = 0.114 if model.startswith('MX') else 0.111

    max_value = 1023 * speed_factor * 6
    value = min(max(value, -max_value), max_value)

    return int(round(direction + abs(value) / (6 * speed_factor), 0))

# MARK: - Torque

def dxl_to_torque(value, model):
    return round(value / 10.23, 1)

def torque_to_dxl(value, model):
    return int(round(value * 10.23, 0))

def dxl_to_load(value, model):
    cw, load = divmod(value, 1024)
    direction = -2 * cw + 1

    return dxl_to_torque(load, model) * direction

# PID Gains

def dxl_to_pid(value, model):
    return (value[0] * 0.004,
            value[1] * 0.48828125,
            value[2] * 0.125)

def pid_to_dxl(value, model):
    truncate = lambda x: int(max(0, min(x, 254)))
    return [truncate(x * y) for x, y in zip(value, (250, 2.048, 8.0))]

# MARK: - Model

dynamixelModels = {
    12: 'AX-12',    # 12 + (0<<8)
    18: 'AX-18',    # 18 + (0<<8)
    28: 'RX-28',    # 28 + (0<<8)
    29: 'MX-28',    # 29 + (0<<8)
    64: 'RX-64',    # 64 + (0<<8)
    310: 'MX-64',   # 54 + (1<<8)
    320: 'MX-106',  # 64 + (1<<8)
}

def dxl_to_model(value, dummy=None):
    return dynamixelModels[value]
# MARK: - Drive Mode

def check_bit(value, offset):
    return bool(value & (1 << offset))

def dxl_to_drive_mode(value, model):
    return ('reverse' if check_bit(value, 0) else 'normal',
            'slave' if check_bit(value, 1) else 'master')

def drive_mode_to_dxl(value, model):
    return (int('slave' in value) << 1 | int('reverse' in value))

# MARK: - Baudrate

dynamixelBaudrates = {
    1: 1000000.0,
    3: 500000.0,
    4: 400000.0,
    7: 250000.0,
    9: 200000.0,
    16: 117647.1,
    34: 57142.9,
    103: 19230.8,
    207: 9615.4,
    250: 2250000.0,
    251: 2500000.0,
    252: 3000000.0,
}

def dxl_to_baudrate(value, model):
    return dynamixelBaudrates[value]

def baudrate_to_dxl(value, model):
    for k, v in dynamixelBaudrates.iteritems():
        if v == value:
            return k
    raise ValueError('incorrect baudrate {} (possible values {})'.format(value, dynamixelBaudrates.values()))

# MARK: - Return Delay Time

def dxl_to_rdt(value, model):
    return value * 2

def rdt_to_dxl(value, model):
    return int(value / 2)

# MARK: - Temperature

def dxl_to_temperature(value, model):
    return float(value)

def temperature_to_dxl(value, model):
    return int(value)

# MARK: - Voltage

def dxl_to_voltage(value, model):
    return value * 0.1

def voltage_to_dxl(value, model):
    return int(value * 10)

# MARK: - Status Return Level

status_level = ('never', 'read', 'always')

def dxl_to_status(value, model):
    return status_level[value]

def status_to_dxl(value, model):
    if value not in status_level:
        raise ValueError('status "{}" should be chosen among {}'.format(value, status_level))
    return status_level.index(value)

# MARK: - Error

dynamixelErrors = ['None Error',
                   'Instruction Error',
                   'Overload Error',
                   'Checksum Error',
                   'Range Error',
                   'Overheating Error',
                   'Angle Limit Error',
                   'Input Voltage Error']

def dxl_to_alarm(value, model):
    return decode_error(value)

def decode_error(error_code):
    bits = numpy.unpackbits(numpy.asarray(error_code, dtype=numpy.uint8))
    return tuple(numpy.array(dynamixelErrors)[bits == 1])

def alarm_to_dxl(value, model):
    if not set(value).issubset(dynamixelErrors):
        raise ValueError('should only contains error among {}'.format(dynamixelErrors))

    indices = [len(dynamixelErrors) - 1 - dynamixelErrors.index(e) for e in value]
    return sum(2 ** i for i in indices)


# MARK: - Various utility functions

def dxl_to_bool(value, model):
    return bool(value)

def bool_to_dxl(value, model):
    return int(value)

def dxl_decode(data):
    if len(data) not in (1, 2):
        raise ValueError('try to decode incorrect data {}'.format(data))

    if len(data) == 1:
        return data[0]

    if len(data) == 2:
        return data[0] + (data[1] << 8)

def dxl_decode_all(data, nb_elem):
    if nb_elem > 1:
        data = list(itertools.izip(*([iter(data)] * (len(data) // nb_elem))))
        return tuple(map(dxl_decode, data))
    else:
        return dxl_decode(data)

def dxl_code(value, length):
    if length not in (1, 2):
        raise ValueError('try to code value with an incorrect length {}'.format(length))

    if length == 1:
        return (value, )

    if length == 2:
        return (value % 256, value >> 8)

def dxl_code_all(value, length, nb_elem):
    if nb_elem > 1:
        return list(itertools.chain(*(dxl_code(v, length) for v in value)))
    else:
        return dxl_code(value, length)



#!/usr/bin/env python

import re
import sys

from setuptools import setup, find_packages


def version():
    with open('pypot/_version.py') as f:
        return re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", f.read()).group(1)

extra = {}
if sys.version_info >= (3,):
    extra['use_2to3'] = True

setup(name='pypot',
      version=version(),
      packages=find_packages(),

      install_requires=['numpy', 'pyserial'],

      extras_require={
          'tools': [],  # Extras require: PyQt4 (not a PyPi packet)
          'doc': ['sphinx', 'sphinx-bootstrap-theme'],
          'server': ['bottle', 'tornado', 'zmq']
      },

      entry_points={
          'gui_scripts': [
              'herborist = pypot.tools.herborist.herborist:main [tools]',
              ],
      },

      setup_requires=['setuptools_git >= 0.3', ],

      include_package_data=True,
      exclude_package_data={'': ['README', '.gitignore']},

      zip_safe=True,

      author='Pierre Rouanet, Matthieu Lapeyre, Haylee Fogg',
      author_email='pierre.rouanet@gmail.com',
      description='Python Library for Robot Control',
      url='https://github.com/poppy-project/pypot',
      license='GNU GENERAL PUBLIC LICENSE Version 3',

      classifiers=[
          "Programming Language :: Python :: 2",
          "Programming Language :: Python :: 3",
          "Topic :: Scientific/Engineering", ],

      **extra
      )



from collections import namedtuple
from operator import attrgetter

Point2D = namedtuple('Point2D', ('x', 'y'))
Point3D = namedtuple('Point3D', ('x', 'y', 'z'))
Point = Point3D

Vector3D = namedtuple('Vector3D', ('x', 'y', 'z'))
Vector = Vector3D

Quaternion = namedtuple('Quaternion', ('x', 'y', 'z', 'w'))

def attrsetter(item):
    def resolve_attr(obj, attr):
        if not attr:
            return obj
        for name in attr.split('.'):
            obj = getattr(obj, name)
        return obj

    def g(obj, value):
        var_path, _, var_name = item.rpartition('.')
        setattr(resolve_attr(obj, var_path), var_name, value)

    return g


import os
import time
import argparse

from numpy import argmax, mean

from pypot.robot import from_json


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('-l', '--log-file',
                        type=str, required=True)
    parser.add_argument('-N', type=int, required=True)
    parser.add_argument('-c', '--config',
                        type=str, required=True)
    args = parser.parse_args()

    bp = args.log_file
    if os.path.exists(bp):
        raise IOError('File already exists: {}'.format(bp))

    robot = from_json(args.config)

    # We keep the controller with most motors connected
    ids = [len(c._ids) for c in robot._dxl_controllers]
    c = robot._dxl_controllers[argmax(ids)]

    print('Using controller with motors {}'.format(c._ids))

    dt = []
    for _ in range(args.N):
        start = time.time()
        c._get_pos_speed_load()
        end = time.time()
        dt.append(end - start)

    # We'll use raw file instead of numpy because of pypy
    with open(bp, 'w') as f:
        f.write(str(dt))

    print('Done in {}ms'.format(mean(dt)*1000))



import threading
import logging
import numpy
import time

from collections import defaultdict
from functools import partial

logger = logging.getLogger(__name__)


class PrimitiveManager(threading.Thread):
    """ Combines all :class:`~pypot.primitive.primitive.Primitive` orders and affect them to the real motors.

        At a predefined frequency, the manager gathers all the orders sent by the primitive to the "fake" motors, combined them thanks to the filter function and affect them to the "real" motors.

        .. note:: The primitives are automatically added (resp. removed) to the manager when they are started (resp. stopped).

        """
    def __init__(self, motors, freq=50, filter=partial(numpy.sum, axis=0)):
        """ :param motors: list of real motors used by the attached primitives
            :type motors: list of :class:`~pypot.dynamixel.motor.DxlMotor`

            :param int freq: update frequency
            :param func filter: function used to combine the different request (default mean)

            """
        threading.Thread.__init__(self, name='Primitive Manager')
        self.daemon = True

        self._prim = []
        self._period = 1.0 / freq
        self._motors = motors
        self._filter = filter
        self._running = threading.Event()
        self._running.set()

    def add(self, p):
        """ Add a primitive to the manager. The primitive automatically attached itself when started. """
        self._prim.append(p)

    def remove(self, p):
        """ Remove a primitive from the manager. The primitive automatically remove itself when stopped. """
        self._prim.remove(p)

    @property
    def primitives(self):
        """ List of all attached :class:`~pypot.primitive.primitive.Primitive`. """
        return self._prim

    def run(self):
        """ Combined at a predefined frequency the request orders and affect them to the real motors.

            .. note:: Should not be called directly but launched through the thread start method.

            """
        while self._running.is_set():
            start = time.time()

            for m in self._motors:
                to_set = defaultdict(list)

                for p in self._prim:
                    for key, val in getattr(p.robot, m.name)._to_set.iteritems():
                        to_set[key].append(val)

                for key, val in to_set.iteritems():
                    filtred_val = self._filter(val)
                    logger.debug('Combined %s.%s from %s to %s',
                                 m.name, key, val, filtred_val)
                    setattr(m, key, filtred_val)

                [p._synced.set() for p in self._prim]

            end = time.time()

            dt = self._period - (end - start)
            if dt > 0:
                time.sleep(dt)

    def stop(self):
        """ Stop the primitive manager. """
        for p in self.primitives[:]:
            p.stop()
            p.wait_to_stop()

        self._running.clear()
        self.join()
